@@ -823,8 +823,7 @@ dir_lookaside_fixup(const CacheKey *key, Vol *d)
       DDebug("dir_lookaside", "fixup %X %X offset %" PRId64 " phase %d %d", key->slice32(0), key->slice32(1),
              dir_offset(&b->new_dir), dir_phase(&b->new_dir), res);
       int64_t o = dir_offset(&b->dir), n = dir_offset(&b->new_dir);
-      d->ram_cache->fixup(key, static_cast<uint32_t>(o >> 32), static_cast<uint32_t>(o), static_cast<uint32_t>(n >> 32),
-                          static_cast<uint32_t>(n));
+      d->ram_cache->fixup(key, static_cast<uint64_t>(o), static_cast<uint64_t>(n));
       d->lookaside[i].remove(b);
       free_EvacuationBlock(b, d->mutex->thread_holding);
       return res;
@@ -1099,8 +1098,7 @@ Lrestart:
       Debug("cache_dir_sync", "sync started");
       /* Don't sync the directory to disk if its not dirty. Syncing the
          clean directory to disk is also the cause of INKqa07151. Increasing
-         the serial serial causes the cache to recover more data
-         than necessary.
+         the serial number causes the cache to recover more data than necessary.
          The dirty bit it set in dir_insert, dir_overwrite and dir_delete_entry
        */
       if (!vol->header->dirty) {
