@@ -43,6 +43,8 @@ struct DiskHandler;
 struct EventIO;
 
 class ServerSessionPool;
+class PreWarmQueue;
+
 class Event;
 class Continuation;
 
@@ -83,6 +85,7 @@ enum ThreadType {
 class EThread : public Thread
 {
 public:
+  static thread_local EThread *this_ethread_ptr;
   /** Handler for tail of event loop.
 
       The event loop should not spin. To avoid that a tail handler is called to block for a limited time.
@@ -118,7 +121,7 @@ public:
 
     @param c Continuation to be called back as soon as possible.
     @param callback_event Event code to be passed back to the
-      continuation's handler. See the the EventProcessor class.
+      continuation's handler. See the EventProcessor class.
     @param cookie User-defined value or pointer to be passed back
       in the Event's object cookie field.
     @return Reference to an Event object representing the scheduling
@@ -282,6 +285,8 @@ public:
   // Set the tail handler.
   void set_tail_handler(LoopTailHandler *handler);
 
+  void set_specific() override;
+
   /* private */
 
   Event *schedule_local(Event *e);
@@ -347,6 +352,7 @@ public:
   Event *start_event = nullptr;
 
   ServerSessionPool *server_session_pool = nullptr;
+  PreWarmQueue *prewarm_queue            = nullptr;
 
   /** Default handler used until it is overridden.
 
