@@ -32,13 +32,19 @@
 
 #include "I_EThread.h"
 #include "I_EventProcessor.h"
+#include <execinfo.h>
 
 const ink_hrtime DELAY_FOR_RETRY = HRTIME_MSECONDS(10);
 
 TS_INLINE Event *
 EThread::schedule_imm(Continuation *cont, int callback_event, void *cookie)
 {
-  Event *e          = ::eventAllocator.alloc();
+  Event *e = ::eventAllocator.alloc();
+
+#ifdef ENABLE_EVENT_TRACKER
+  e->set_location();
+#endif
+
   e->callback_event = callback_event;
   e->cookie         = cookie;
   return schedule(e->init(cont, 0, 0));
@@ -47,7 +53,12 @@ EThread::schedule_imm(Continuation *cont, int callback_event, void *cookie)
 TS_INLINE Event *
 EThread::schedule_at(Continuation *cont, ink_hrtime t, int callback_event, void *cookie)
 {
-  Event *e          = ::eventAllocator.alloc();
+  Event *e = ::eventAllocator.alloc();
+
+#ifdef ENABLE_EVENT_TRACKER
+  e->set_location();
+#endif
+
   e->callback_event = callback_event;
   e->cookie         = cookie;
   return schedule(e->init(cont, t, 0));
@@ -56,7 +67,12 @@ EThread::schedule_at(Continuation *cont, ink_hrtime t, int callback_event, void
 TS_INLINE Event *
 EThread::schedule_in(Continuation *cont, ink_hrtime t, int callback_event, void *cookie)
 {
-  Event *e          = ::eventAllocator.alloc();
+  Event *e = ::eventAllocator.alloc();
+
+#ifdef ENABLE_EVENT_TRACKER
+  e->set_location();
+#endif
+
   e->callback_event = callback_event;
   e->cookie         = cookie;
   return schedule(e->init(cont, get_hrtime() + t, 0));
@@ -65,7 +81,12 @@ EThread::schedule_in(Continuation *cont, ink_hrtime t, int callback_event, void
 TS_INLINE Event *
 EThread::schedule_every(Continuation *cont, ink_hrtime t, int callback_event, void *cookie)
 {
-  Event *e          = ::eventAllocator.alloc();
+  Event *e = ::eventAllocator.alloc();
+
+#ifdef ENABLE_EVENT_TRACKER
+  e->set_location();
+#endif
+
   e->callback_event = callback_event;
   e->cookie         = cookie;
   if (t < 0) {
@@ -79,7 +100,10 @@ TS_INLINE Event *
 EThread::schedule(Event *e)
 {
   e->ethread = this;
-  ink_assert(tt == REGULAR);
+  if (tt != REGULAR) {
+    ink_assert(tt == DEDICATED);
+    return eventProcessor.schedule(e, ET_CALL);
+  }
   if (e->continuation->mutex) {
     e->mutex = e->continuation->mutex;
   } else {
@@ -104,7 +128,12 @@ EThread::schedule(Event *e)
 TS_INLINE Event *
 EThread::schedule_imm_local(Continuation *cont, int callback_event, void *cookie)
 {
-  Event *e          = EVENT_ALLOC(eventAllocator, this);
+  Event *e = EVENT_ALLOC(eventAllocator, this);
+
+#ifdef ENABLE_EVENT_TRACKER
+  e->set_location();
+#endif
+
   e->callback_event = callback_event;
   e->cookie         = cookie;
   return schedule_local(e->init(cont, 0, 0));
@@ -113,7 +142,12 @@ EThread::schedule_imm_local(Continuation *cont, int callback_event, void *cookie
 TS_INLINE Event *
 EThread::schedule_at_local(Continuation *cont, ink_hrtime t, int callback_event, void *cookie)
 {
-  Event *e          = EVENT_ALLOC(eventAllocator, this);
+  Event *e = EVENT_ALLOC(eventAllocator, this);
+
+#ifdef ENABLE_EVENT_TRACKER
+  e->set_location();
+#endif
+
   e->callback_event = callback_event;
   e->cookie         = cookie;
   return schedule_local(e->init(cont, t, 0));
@@ -122,7 +156,12 @@ EThread::schedule_at_local(Continuation *cont, ink_hrtime t, int callback_event,
 TS_INLINE Event *
 EThread::schedule_in_local(Continuation *cont, ink_hrtime t, int callback_event, void *cookie)
 {
-  Event *e          = EVENT_ALLOC(eventAllocator, this);
+  Event *e = EVENT_ALLOC(eventAllocator, this);
+
+#ifdef ENABLE_EVENT_TRACKER
+  e->set_location();
+#endif
+
   e->callback_event = callback_event;
   e->cookie         = cookie;
   return schedule_local(e->init(cont, get_hrtime() + t, 0));
@@ -131,7 +170,12 @@ EThread::schedule_in_local(Continuation *cont, ink_hrtime t, int callback_event,
 TS_INLINE Event *
 EThread::schedule_every_local(Continuation *cont, ink_hrtime t, int callback_event, void *cookie)
 {
-  Event *e          = EVENT_ALLOC(eventAllocator, this);
+  Event *e = EVENT_ALLOC(eventAllocator, this);
+
+#ifdef ENABLE_EVENT_TRACKER
+  e->set_location();
+#endif
+
   e->callback_event = callback_event;
   e->cookie         = cookie;
   if (t < 0) {
@@ -183,7 +227,7 @@ EThread::schedule_spawn(Continuation *c, int ev, void *cookie)
 TS_INLINE EThread *
 this_ethread()
 {
-  return static_cast<EThread *>(this_thread());
+  return EThread::this_ethread_ptr;
 }
 
 TS_INLINE EThread *
