@@ -51,6 +51,26 @@ int const EThread::SAMPLE_COUNT[N_EVENT_TIMESCALES] = {10, 100, 1000};
 
 int thread_max_heartbeat_mseconds = THREAD_MAX_HEARTBEAT_MSECONDS;
 
+// To define a class inherits from Thread:
+//   1) Define an independent thread_local static member
+//   2) Override Thread::set_specific() and assign that member and call Thread::set_specific()
+//   3) Define this_Xthread() which get thread specific data
+//   4) Clear thread specific data at destructor function.
+//
+// The below comments are copied from I_Thread.h
+//
+// Additionally, the EThread class (derived from Thread) maintains its
+// own independent data. All (and only) the threads created in the Event
+// Subsystem have this data.
+thread_local EThread *EThread::this_ethread_ptr;
+
+void
+EThread::set_specific()
+{
+  this_ethread_ptr = this;
+  Thread::set_specific();
+}
+
 EThread::EThread()
 {
   memset(thread_private, 0, PER_THREAD_DATA);
@@ -92,7 +112,13 @@ EThread::EThread(ThreadType att, Event *e) : tt(att), start_event(e)
 
 // Provide a destructor so that SDK functions which create and destroy
 // threads won't have to deal with EThread memory deallocation.
-EThread::~EThread() {}
+EThread::~EThread()
+{
+  ink_release_assert(mutex->thread_holding == static_cast<EThread *>(this));
+  if (this_ethread_ptr == this) {
+    this_ethread_ptr = nullptr;
+  }
+}
 
 bool
 EThread::is_event_type(EventType et)
