@@ -48,6 +48,8 @@
 #include "P_UnixNet.h"
 #include "P_ALPNSupport.h"
 #include "TLSSessionResumptionSupport.h"
+#include "TLSSNISupport.h"
+#include "TLSBasicSupport.h"
 #include "P_SSLUtils.h"
 #include "P_SSLConfig.h"
 
@@ -93,7 +95,11 @@ enum SSLHandshakeStatus { SSL_HANDSHAKE_ONGOING, SSL_HANDSHAKE_DONE, SSL_HANDSHA
 //  A VConnection for a network socket.
 //
 //////////////////////////////////////////////////////////////////
-class SSLNetVConnection : public UnixNetVConnection, public ALPNSupport, public TLSSessionResumptionSupport
+class SSLNetVConnection : public UnixNetVConnection,
+                          public ALPNSupport,
+                          public TLSSessionResumptionSupport,
+                          public TLSSNISupport,
+                          public TLSBasicSupport
 {
   typedef UnixNetVConnection super; ///< Parent type.
 
@@ -102,19 +108,12 @@ public:
   void clear() override;
   void free(EThread *t) override;
 
-  virtual void
-  enableRead()
-  {
-    read.enabled  = 1;
-    write.enabled = 1;
-  }
-
   bool
   trackFirstHandshake() override
   {
-    bool retval = sslHandshakeBeginTime == 0;
+    bool retval = this->get_tls_handshake_begin_time() == 0;
     if (retval) {
-      sslHandshakeBeginTime = Thread::get_hrtime();
+      this->_record_tls_handshake_begin_time();
     }
     return retval;
   }
@@ -144,9 +143,6 @@ public:
   ////////////////////////////////////////////////////////////
   SSLNetVConnection();
   ~SSLNetVConnection() override {}
-  static int advertise_next_protocol(SSL *ssl, const unsigned char **out, unsigned *outlen, void *);
-  static int select_next_protocol(SSL *ssl, const unsigned char **out, unsigned char *outlen, const unsigned char *in,
-                                  unsigned inlen, void *);
 
   bool
   getSSLClientRenegotiationAbort() const
@@ -281,43 +277,6 @@ public:
     return retval;
   }
 
-  const char *
-  getSSLProtocol() const
-  {
-    return ssl ? SSL_get_version(ssl) : nullptr;
-  }
-
-  const char *
-  getSSLCipherSuite() const
-  {
-    return ssl ? SSL_get_cipher_name(ssl) : nullptr;
-  }
-
-  const char *
-  getSSLCurve() const
-  {
-    if (!ssl) {
-      return nullptr;
-    }
-    ssl_curve_id curve;
-    if (getSSLSessionCacheHit()) {
-      curve = getSSLCurveNID();
-    } else {
-      curve = SSLGetCurveNID(ssl);
-    }
-#ifndef OPENSSL_IS_BORINGSSL
-    if (curve == NID_undef) {
-      return nullptr;
-    }
-    return OBJ_nid2sn(curve);
-#else
-    if (curve == 0) {
-      return nullptr;
-    }
-    return SSL_get_curve_name(curve);
-#endif
-  }
-
   bool
   has_tunnel_destination() const
   {
@@ -336,25 +295,17 @@ public:
     return tunnel_port;
   }
 
-  /* Returns true if this vc was configured for forward_route or partial_blind_route
-   */
-  bool
-  decrypt_tunnel()
-  {
-    return has_tunnel_destination() && tunnel_decrypt;
-  }
-
-  /* Returns true if this vc was configured partial_blind_route
-   */
-  bool
-  upstream_tls()
-  {
-    return has_tunnel_destination() && tls_upstream;
-  }
+  bool decrypt_tunnel() const;
+  bool upstream_tls() const;
+  SNIRoutingType tunnel_type() const;
+  YamlSNIConfig::TunnelPreWarm tunnel_prewarm() const;
 
   void
-  set_tunnel_destination(const std::string_view &destination, bool decrypt, bool upstream_tls)
+  set_tunnel_destination(const std::string_view &destination, SNIRoutingType type, YamlSNIConfig::TunnelPreWarm prewarm)
   {
+    _tunnel_type    = type;
+    _tunnel_prewarm = prewarm;
+
     auto pos = destination.find(":");
     if (nullptr != tunnel_host) {
       ats_free(tunnel_host);
@@ -366,36 +317,32 @@ public:
       tunnel_port = 0;
       tunnel_host = ats_strndup(destination.data(), destination.length());
     }
-    tunnel_decrypt = decrypt;
-    tls_upstream   = upstream_tls;
   }
 
   int populate_protocol(std::string_view *results, int n) const override;
   const char *protocol_contains(std::string_view tag) const override;
 
   /**
-   * Populate the current object based on the socket information in in the
+   * Populate the current object based on the socket information in the
    * con parameter and the ssl object in the arg parameter
    * This is logic is invoked when the NetVC object is created in a new thread context
    */
   int populate(Connection &con, Continuation *c, void *arg) override;
 
-  SSL *ssl                         = nullptr;
-  ink_hrtime sslHandshakeBeginTime = 0;
-  ink_hrtime sslHandshakeEndTime   = 0;
-  ink_hrtime sslLastWriteTime      = 0;
-  int64_t sslTotalBytesSent        = 0;
+  SSL *ssl                    = nullptr;
+  ink_hrtime sslLastWriteTime = 0;
+  int64_t sslTotalBytesSent   = 0;
+
+  std::shared_ptr<SSL_SESSION> client_sess = nullptr;
 
   // The serverName is either a pointer to the (null-terminated) name fetched from the
   // SSL object or the empty string.
   const char *
   get_server_name() const override
   {
-    return _serverName.get() ? _serverName.get() : "";
+    return _get_sni_server_name() ? _get_sni_server_name() : "";
   }
 
-  void set_server_name(std::string_view name);
-
   bool
   support_sni() const override
   {
@@ -480,20 +427,39 @@ public:
     return _ca_cert_dir.get();
   }
 
+  void
+  set_valid_tls_protocols(unsigned long proto_mask, unsigned long max_mask)
+  {
+    SSL_set_options(this->ssl, proto_mask);
+    SSL_clear_options(this->ssl, max_mask & ~proto_mask);
+  }
+
 protected:
+  SSL *
+  _get_ssl_object() const override
+  {
+    return this->ssl;
+  }
+  ssl_curve_id _get_tls_curve() const override;
+
   const IpEndpoint &
   _getLocalEndpoint() override
   {
     return local_addr;
   }
 
+  void _fire_ssl_servername_event() override;
+
 private:
   std::string_view map_tls_protocol_to_tag(const char *proto_string) const;
   bool update_rbio(bool move_to_socket);
   void increment_ssl_version_metric(int version) const;
+  NetProcessor *_getNetProcessor() override;
+  void *_prepareForMigration() override;
 
   enum SSLHandshakeStatus sslHandshakeStatus = SSL_HANDSHAKE_ONGOING;
   bool sslClientRenegotiationAbort           = false;
+  bool first_ssl_connect                     = true;
   MIOBuffer *handShakeBuffer                 = nullptr;
   IOBufferReader *handShakeHolder            = nullptr;
   IOBufferReader *handShakeReader            = nullptr;
@@ -523,21 +489,66 @@ private:
     HANDSHAKE_HOOKS_DONE
   } sslHandshakeHookState = HANDSHAKE_HOOKS_PRE;
 
-  int64_t redoWriteSize       = 0;
-  char *tunnel_host           = nullptr;
-  in_port_t tunnel_port       = 0;
-  bool tunnel_decrypt         = false;
-  bool tls_upstream           = false;
+  int64_t redoWriteSize = 0;
+
+  char *tunnel_host                            = nullptr;
+  in_port_t tunnel_port                        = 0;
+  SNIRoutingType _tunnel_type                  = SNIRoutingType::NONE;
+  YamlSNIConfig::TunnelPreWarm _tunnel_prewarm = YamlSNIConfig::TunnelPreWarm::UNSET;
+
   X509_STORE_CTX *verify_cert = nullptr;
 
   // Null-terminated string, or nullptr if there is no SNI server name.
-  std::unique_ptr<char[]> _serverName;
   std::unique_ptr<char[]> _ca_cert_file;
   std::unique_ptr<char[]> _ca_cert_dir;
 
   EventIO async_ep{};
+
+private:
+  void _make_ssl_connection(SSL_CTX *ctx);
+  void _bindSSLObject();
+  void _unbindSSLObject();
+
+  int _ssl_read_from_net(EThread *lthread, int64_t &ret);
+  ssl_error_t _ssl_read_buffer(void *buf, int64_t nbytes, int64_t &nread);
+  ssl_error_t _ssl_write_buffer(const void *buf, int64_t nbytes, int64_t &nwritten);
+  ssl_error_t _ssl_connect();
+  ssl_error_t _ssl_accept();
 };
 
 typedef int (SSLNetVConnection::*SSLNetVConnHandler)(int, void *);
 
 extern ClassAllocator<SSLNetVConnection> sslNetVCAllocator;
+
+//
+// Inline Functions
+//
+inline SNIRoutingType
+SSLNetVConnection::tunnel_type() const
+{
+  return _tunnel_type;
+}
+
+inline YamlSNIConfig::TunnelPreWarm
+SSLNetVConnection::tunnel_prewarm() const
+{
+  return _tunnel_prewarm;
+}
+
+/**
+   Returns true if this vc was configured for forward_route or partial_blind_route
+ */
+inline bool
+SSLNetVConnection::decrypt_tunnel() const
+{
+  return _tunnel_type == SNIRoutingType::FORWARD || _tunnel_type == SNIRoutingType::PARTIAL_BLIND;
+}
+
+/**
+   Returns true if this vc was configured partial_blind_route
+ */
+inline bool
+SSLNetVConnection::upstream_tls() const
+{
+  return _tunnel_type == SNIRoutingType::PARTIAL_BLIND;
+}
