@@ -25,11 +25,12 @@
   recording log entries. See the header file LogBuffer.h for more
   information on the structure of a LogBuffer.
  */
-#include "ts/ink_platform.h"
+#include "tscore/ink_platform.h"
+#include "tscore/BufferWriter.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 
 #include "P_EventSystem.h"
 #include "LogField.h"
@@ -37,7 +38,6 @@
 #include "LogFormat.h"
 #include "LogUtils.h"
 #include "LogFile.h"
-#include "LogHost.h"
 #include "LogObject.h"
 #include "LogAccess.h"
 #include "LogConfig.h"
@@ -55,29 +55,19 @@ enum {
 
 FieldListCacheElement fieldlist_cache[FIELDLIST_CACHE_SIZE];
 int fieldlist_cache_entries = 0;
-vint32 LogBuffer::M_ID      = 0;
+int32_t LogBuffer::M_ID;
 
 /*-------------------------------------------------------------------------
   The following LogBufferHeader routines are used to grab strings out from
   the data section using the offsets held in the buffer header.
   -------------------------------------------------------------------------*/
 
-char *
-LogBufferHeader::fmt_name()
-{
-  char *addr = NULL;
-  if (fmt_name_offset) {
-    addr = (char *)this + fmt_name_offset;
-  }
-  return addr;
-}
-
 char *
 LogBufferHeader::fmt_fieldlist()
 {
-  char *addr = NULL;
+  char *addr = nullptr;
   if (fmt_fieldlist_offset) {
-    addr = (char *)this + fmt_fieldlist_offset;
+    addr = reinterpret_cast<char *>(this) + fmt_fieldlist_offset;
   }
   return addr;
 }
@@ -85,9 +75,9 @@ LogBufferHeader::fmt_fieldlist()
 char *
 LogBufferHeader::fmt_printf()
 {
-  char *addr = NULL;
+  char *addr = nullptr;
   if (fmt_printf_offset) {
-    addr = (char *)this + fmt_printf_offset;
+    addr = reinterpret_cast<char *>(this) + fmt_printf_offset;
   }
   return addr;
 }
@@ -95,9 +85,9 @@ LogBufferHeader::fmt_printf()
 char *
 LogBufferHeader::src_hostname()
 {
-  char *addr = NULL;
+  char *addr = nullptr;
   if (src_hostname_offset) {
-    addr = (char *)this + src_hostname_offset;
+    addr = reinterpret_cast<char *>(this) + src_hostname_offset;
   }
   return addr;
 }
@@ -105,22 +95,14 @@ LogBufferHeader::src_hostname()
 char *
 LogBufferHeader::log_filename()
 {
-  char *addr = NULL;
+  char *addr = nullptr;
   if (log_filename_offset) {
-    addr = (char *)this + log_filename_offset;
+    addr = reinterpret_cast<char *>(this) + log_filename_offset;
   }
   return addr;
 }
 
-/*-------------------------------------------------------------------------
-  LogBuffer::LogBuffer
-
-  Initialize a LogBuffer object, which is just an AbstractBuffer object
-  with the addition of a pointer for keeping track of the LogObject object
-  that is allocating this buffer.
-  -------------------------------------------------------------------------*/
-
-LogBuffer::LogBuffer(LogObject *owner, size_t size, size_t buf_align, size_t write_align)
+LogBuffer::LogBuffer(const LogConfig *cfg, LogObject *owner, size_t size, size_t buf_align, size_t write_align)
   : m_size(size), m_buf_align(buf_align), m_write_align(write_align), m_owner(owner), m_references(0)
 {
   size_t hdr_size;
@@ -129,33 +111,33 @@ LogBuffer::LogBuffer(LogObject *owner, size_t size, size_t buf_align, size_t wri
   //
   int64_t alloc_size = size + buf_align;
 
-  if (alloc_size <= max_iobuffer_size) {
-    m_buffer_fast_allocator_size = buffer_size_to_index(alloc_size);
-    m_unaligned_buffer           = (char *)ioBufAllocator[m_buffer_fast_allocator_size].alloc_void();
+  if (alloc_size <= BUFFER_SIZE_FOR_INDEX(cfg->logbuffer_max_iobuf_index)) {
+    m_buffer_fast_allocator_size = buffer_size_to_index(alloc_size, cfg->logbuffer_max_iobuf_index);
+    m_unaligned_buffer           = static_cast<char *>(ioBufAllocator[m_buffer_fast_allocator_size].alloc_void());
   } else {
     m_buffer_fast_allocator_size = -1;
-    m_unaligned_buffer           = (char *)ats_malloc(alloc_size);
+    m_unaligned_buffer           = static_cast<char *>(ats_malloc(alloc_size));
   }
-  m_buffer = (char *)align_pointer_forward(m_unaligned_buffer, buf_align);
+  m_buffer = static_cast<char *>(align_pointer_forward(m_unaligned_buffer, buf_align));
 
   // add the header
-  hdr_size = _add_buffer_header();
+  hdr_size = _add_buffer_header(cfg);
 
   // initialize buffer state
   m_state.s.offset = hdr_size;
 
   // update the buffer id (m_id gets the old value)
-  m_id = (uint32_t)ink_atomic_increment((pvint32)&M_ID, 1);
+  m_id = static_cast<uint32_t>(ink_atomic_increment(&M_ID, 1));
 
-  m_expiration_time = LogUtils::timestamp() + Log::config->max_secs_per_buffer;
+  m_expiration_time = LogUtils::timestamp() + cfg->max_secs_per_buffer;
 
   Debug("log-logbuffer", "[%p] Created buffer %u for %s at address %p, size %d", this_ethread(), m_id, m_owner->get_base_filename(),
         m_buffer, (int)size);
 }
 
 LogBuffer::LogBuffer(LogObject *owner, LogBufferHeader *header)
-  : m_unaligned_buffer(NULL),
-    m_buffer((char *)header),
+  : m_unaligned_buffer(nullptr),
+    m_buffer(reinterpret_cast<char *>(header)),
     m_size(0),
     m_buf_align(LB_DEFAULT_ALIGN),
     m_write_align(INK_MIN_ALIGN),
@@ -172,7 +154,7 @@ LogBuffer::LogBuffer(LogObject *owner, LogBufferHeader *header)
 
   // update the buffer id (m_id gets the old value)
   //
-  m_id = (uint32_t)ink_atomic_increment((pvint32)&M_ID, 1);
+  m_id = static_cast<uint32_t>(ink_atomic_increment(&M_ID, 1));
 
   Debug("log-logbuffer", "[%p] Created repurposed buffer %u for %s at address %p", this_ethread(), m_id,
         m_owner->get_base_filename(), m_buffer);
@@ -181,7 +163,7 @@ LogBuffer::LogBuffer(LogObject *owner, LogBufferHeader *header)
 void
 LogBuffer::freeLogBuffer()
 {
-  char *log_buffer = NULL;
+  char *log_buffer = nullptr;
 
   if (m_unaligned_buffer) {
     log_buffer = m_unaligned_buffer;
@@ -201,8 +183,8 @@ LogBuffer::freeLogBuffer()
 LogBuffer::~LogBuffer()
 {
   freeLogBuffer();
-  m_buffer           = 0;
-  m_unaligned_buffer = 0;
+  m_buffer           = nullptr;
+  m_unaligned_buffer = nullptr;
 }
 
 /*-------------------------------------------------------------------------
@@ -225,9 +207,11 @@ LogBuffer::checkout_write(size_t *write_offset, size_t write_size)
   size_t offset            = 0;
   size_t actual_write_size = INK_ALIGN(write_size + sizeof(LogEntryHeader), m_write_align);
 
-  uint64_t retries = (uint64_t)-1;
+  uint64_t retries = static_cast<uint64_t>(-1);
   do {
-    new_s = old_s = m_state;
+    // we want sequence points between these two statements
+    old_s = m_state;
+    new_s = old_s;
 
     if (old_s.s.full) {
       // the buffer has already been set to full by somebody else
@@ -276,7 +260,7 @@ LogBuffer::checkout_write(size_t *write_offset, size_t write_size)
       }
 
       if (switch_state(old_s, new_s)) {
-        // we succeded in setting the new state
+        // we succeeded in setting the new state
         break;
       }
     }
@@ -292,7 +276,7 @@ LogBuffer::checkout_write(size_t *write_offset, size_t write_size)
     // ink_release_assert(mutex->thread_holding == this_ethread());
     // SUM_DYN_STAT(log_stat_bytes_buffered_stat, actual_write_size);
 
-    LogEntryHeader *entry_header = (LogEntryHeader *)&m_buffer[offset];
+    LogEntryHeader *entry_header = reinterpret_cast<LogEntryHeader *>(&m_buffer[offset]);
     // entry_header->timestamp = LogUtils::timestamp();
     struct timeval tp = ink_gettimeofday();
 
@@ -361,7 +345,7 @@ LogBuffer::add_header_str(const char *str, char *buf_ptr, unsigned buf_len)
 }
 
 size_t
-LogBuffer::_add_buffer_header()
+LogBuffer::_add_buffer_header(const LogConfig *cfg)
 {
   size_t header_len;
 
@@ -369,7 +353,7 @@ LogBuffer::_add_buffer_header()
   // initialize the header
   //
   LogFormat *fmt                 = m_owner->m_format;
-  m_header                       = (LogBufferHeader *)m_buffer;
+  m_header                       = reinterpret_cast<LogBufferHeader *>(m_buffer);
   m_header->cookie               = LOG_SEGMENT_COOKIE;
   m_header->version              = LOG_SEGMENT_VERSION;
   m_header->format_type          = fmt->type();
@@ -408,9 +392,9 @@ LogBuffer::_add_buffer_header()
     m_header->fmt_printf_offset = header_len;
     header_len += add_header_str(fmt->printf_str(), &m_buffer[header_len], m_size - header_len);
   }
-  if (Log::config->hostname) {
+  if (cfg->hostname) {
     m_header->src_hostname_offset = header_len;
-    header_len += add_header_str(Log::config->hostname, &m_buffer[header_len], m_size - header_len);
+    header_len += add_header_str(cfg->hostname, &m_buffer[header_len], m_size - header_len);
   }
   if (m_owner->get_base_filename()) {
     m_header->log_filename_offset = header_len;
@@ -461,20 +445,20 @@ LogBuffer::max_entry_bytes()
 int
 LogBuffer::resolve_custom_entry(LogFieldList *fieldlist, char *printf_str, char *read_from, char *write_to, int write_to_len,
                                 long timestamp, long timestamp_usec, unsigned buffer_version, LogFieldList *alt_fieldlist,
-                                char *alt_printf_str)
+                                char *alt_printf_str, LogEscapeType escape_type)
 {
-  if (fieldlist == NULL || printf_str == NULL) {
+  if (fieldlist == nullptr || printf_str == nullptr) {
     return 0;
   }
 
-  int *readfrom_map = NULL;
+  int *readfrom_map = nullptr;
 
   if (alt_fieldlist && alt_printf_str) {
     LogField *f, *g;
     int n_alt_fields = alt_fieldlist->count();
     int i            = 0;
 
-    readfrom_map = (int *)ats_malloc(n_alt_fields * sizeof(int));
+    readfrom_map = static_cast<int *>(ats_malloc(n_alt_fields * sizeof(int)));
     for (f = alt_fieldlist->first(); f; f = alt_fieldlist->next(f)) {
       int readfrom_pos = 0;
       bool found_match = false;
@@ -502,9 +486,11 @@ LogBuffer::resolve_custom_entry(LogFieldList *fieldlist, char *printf_str, char
   // LogField object, obtained from the fieldlist.
   //
 
-  LogField *field   = fieldlist->first();
-  int printf_len    = (int)::strlen(printf_str); // OPTIMIZE
-  int bytes_written = 0;
+  LogField *field     = fieldlist->first();
+  LogField *lastField = nullptr;                                // For debug message.
+  int markCount       = 0;                                      // For debug message.
+  int printf_len      = static_cast<int>(::strlen(printf_str)); // OPTIMIZE
+  int bytes_written   = 0;
   int res, i;
 
   const char *buffer_size_exceeded_msg = "Traffic Server is skipping the current log entry because its size "
@@ -512,113 +498,30 @@ LogBuffer::resolve_custom_entry(LogFieldList *fieldlist, char *printf_str, char
 
   for (i = 0; i < printf_len; i++) {
     if (printf_str[i] == LOG_FIELD_MARKER) {
-      if (field != NULL) {
+      ++markCount;
+      if (field != nullptr) {
         char *to = &write_to[bytes_written];
-
-        // for timestamps that are not aggregates, we take the
-        // value from the function argument;  otherwise we use the
-        // unmarshaling function
-        bool non_aggregate_timestamp = false;
-
-        if (field->aggregate() == LogField::NO_AGGREGATE) {
-          const char *sym = field->symbol();
-
-          if (strcmp(sym, "cqts") == 0) {
-            char *ptr = (char *)&timestamp;
-            res       = LogAccess::unmarshal_int_to_str(&ptr, to, write_to_len - bytes_written);
-            if (buffer_version > 1) {
-              // space was reserved in read buffer; remove it
-              read_from += INK_MIN_ALIGN;
-            }
-
-            non_aggregate_timestamp = true;
-
-          } else if (strcmp(sym, "cqth") == 0) {
-            char *ptr = (char *)&timestamp;
-            res       = LogAccess::unmarshal_int_to_str_hex(&ptr, to, write_to_len - bytes_written);
-            if (buffer_version > 1) {
-              // space was reserved in read buffer; remove it
-              read_from += INK_MIN_ALIGN;
-            }
-
-            non_aggregate_timestamp = true;
-
-          } else if (strcmp(sym, "cqtq") == 0) {
-            // From lib/ts
-            res = squid_timestamp_to_buf(to, write_to_len - bytes_written, timestamp, timestamp_usec);
-            if (res < 0) {
-              res = -1;
-            }
-
-            if (buffer_version > 1) {
-              // space was reserved in read buffer; remove it
-              read_from += INK_MIN_ALIGN;
-            }
-
-            non_aggregate_timestamp = true;
-
-          } else if (strcmp(sym, "cqtn") == 0) {
-            char *str = LogUtils::timestamp_to_netscape_str(timestamp);
-            res       = (int)::strlen(str);
-            if (res < write_to_len - bytes_written) {
-              memcpy(to, str, res);
-            } else {
-              res = -1;
-            }
-            if (buffer_version > 1) {
-              // space was reserved in read buffer; remove it
-              read_from += INK_MIN_ALIGN;
-            }
-
-            non_aggregate_timestamp = true;
-
-          } else if (strcmp(sym, "cqtd") == 0) {
-            char *str = LogUtils::timestamp_to_date_str(timestamp);
-            res       = (int)::strlen(str);
-            if (res < write_to_len - bytes_written) {
-              memcpy(to, str, res);
-            } else {
-              res = -1;
-            }
-            if (buffer_version > 1) {
-              // space was reserved in read buffer; remove it
-              read_from += INK_MIN_ALIGN;
-            }
-
-            non_aggregate_timestamp = true;
-
-          } else if (strcmp(sym, "cqtt") == 0) {
-            char *str = LogUtils::timestamp_to_time_str(timestamp);
-            res       = (int)::strlen(str);
-            if (res < write_to_len - bytes_written) {
-              memcpy(to, str, res);
-            } else {
-              res = -1;
-            }
-            if (buffer_version > 1) {
-              // space was reserved in read buffer; remove it
-              read_from += INK_MIN_ALIGN;
-            }
-
-            non_aggregate_timestamp = true;
-          }
-        }
-
-        if (!non_aggregate_timestamp) {
-          res = field->unmarshal(&read_from, to, write_to_len - bytes_written);
-        }
+        res      = field->unmarshal(&read_from, to, write_to_len - bytes_written, escape_type);
 
         if (res < 0) {
-          Note("%s", buffer_size_exceeded_msg);
+          SiteThrottledNote("%s", buffer_size_exceeded_msg);
           bytes_written = 0;
           break;
         }
 
         bytes_written += res;
-        field = fieldlist->next(field);
+        lastField = field;
+        field     = fieldlist->next(field);
       } else {
-        Note("There are more field markers than fields;"
-             " cannot process log entry");
+        ts::LocalBufferWriter<10 * 1024> bw;
+        if (auto bs = fieldlist->badSymbols(); bs.size() > 0) {
+          bw << " (likely due to bad symbols " << bs << " in log format)";
+        }
+        Note("There are more field markers than fields%*s;"
+             " cannot process log entry '%.*s'. Last field = '%s' printf_str='%s' pos=%d/%d count=%d alt_printf_str='%s'",
+             static_cast<int>(bw.size()), bw.data(), bytes_written, write_to, lastField == nullptr ? "*" : lastField->symbol(),
+             printf_str == nullptr ? "*NULL*" : printf_str, i, printf_len, markCount,
+             alt_printf_str == nullptr ? "*NULL*" : alt_printf_str);
         bytes_written = 0;
         break;
       }
@@ -626,7 +529,7 @@ LogBuffer::resolve_custom_entry(LogFieldList *fieldlist, char *printf_str, char
       if (1 + bytes_written < write_to_len) {
         write_to[bytes_written++] = printf_str[i];
       } else {
-        Note("%s", buffer_size_exceeded_msg);
+        SiteThrottledNote("%s", buffer_size_exceeded_msg);
         bytes_written = 0;
         break;
       }
@@ -646,16 +549,16 @@ LogBuffer::resolve_custom_entry(LogFieldList *fieldlist, char *printf_str, char
   -------------------------------------------------------------------------*/
 int
 LogBuffer::to_ascii(LogEntryHeader *entry, LogFormatType type, char *buf, int buf_len, const char *symbol_str, char *printf_str,
-                    unsigned buffer_version, const char *alt_format)
+                    unsigned buffer_version, const char *alt_format, LogEscapeType escape_type)
 {
-  ink_assert(entry != NULL);
+  ink_assert(entry != nullptr);
   ink_assert(type == LOG_FORMAT_CUSTOM || type == LOG_FORMAT_TEXT);
-  ink_assert(buf != NULL);
+  ink_assert(buf != nullptr);
 
   char *read_from; // keeps track of where we're reading from entry
   char *write_to;  // keeps track of where we're writing into buf
 
-  read_from = (char *)entry + sizeof(LogEntryHeader);
+  read_from = reinterpret_cast<char *>(entry) + sizeof(LogEntryHeader);
   write_to  = buf;
 
   if (type == LOG_FORMAT_TEXT) {
@@ -677,7 +580,7 @@ LogBuffer::to_ascii(LogEntryHeader *entry, LogFormatType type, char *buf, int bu
   //
 
   int i;
-  LogFieldList *fieldlist = NULL;
+  LogFieldList *fieldlist = nullptr;
   bool delete_fieldlist_p = false; // need to free the fieldlist?
 
   for (i = 0; i < fieldlist_cache_entries; i++) {
@@ -691,7 +594,7 @@ LogBuffer::to_ascii(LogEntryHeader *entry, LogFormatType type, char *buf, int bu
   if (!fieldlist) {
     Debug("log-fieldlist", "Fieldlist for %s not found; creating ...", symbol_str);
     fieldlist = new LogFieldList;
-    ink_assert(fieldlist != NULL);
+    ink_assert(fieldlist != nullptr);
     bool contains_aggregates = false;
     LogFormat::parse_symbol_string(symbol_str, fieldlist, &contains_aggregates);
 
@@ -705,9 +608,9 @@ LogBuffer::to_ascii(LogEntryHeader *entry, LogFormatType type, char *buf, int bu
     }
   }
 
-  LogFieldList *alt_fieldlist = NULL;
-  char *alt_printf_str        = NULL;
-  char *alt_symbol_str        = NULL;
+  LogFieldList *alt_fieldlist = nullptr;
+  char *alt_printf_str        = nullptr;
+  char *alt_symbol_str        = nullptr;
   bool bad_alt_format         = false;
 
   if (alt_format) {
@@ -733,13 +636,13 @@ LogBuffer::to_ascii(LogEntryHeader *entry, LogFormatType type, char *buf, int bu
     delete alt_fieldlist;
     ats_free(alt_printf_str);
     ats_free(alt_symbol_str);
-    alt_fieldlist  = NULL;
-    alt_printf_str = NULL;
-    alt_symbol_str = NULL;
+    alt_fieldlist  = nullptr;
+    alt_printf_str = nullptr;
+    alt_symbol_str = nullptr;
   }
 
   int ret = resolve_custom_entry(fieldlist, printf_str, read_from, write_to, buf_len, entry->timestamp, entry->timestamp_usec,
-                                 buffer_version, alt_fieldlist, alt_printf_str);
+                                 buffer_version, alt_fieldlist, alt_printf_str, escape_type);
 
   delete alt_fieldlist;
   ats_free(alt_printf_str);
@@ -770,7 +673,7 @@ LogBuffer::to_ascii(LogEntryHeader *entry, LogFormatType type, char *buf, int bu
 LogBufferList::LogBufferList()
 {
   m_size = 0;
-  ink_mutex_init(&m_mutex, "LogBufferList");
+  ink_mutex_init(&m_mutex);
 }
 
 /*-------------------------------------------------------------------------
@@ -780,7 +683,7 @@ LogBufferList::LogBufferList()
 LogBufferList::~LogBufferList()
 {
   LogBuffer *lb;
-  while ((lb = get()) != NULL) {
+  while ((lb = get()) != nullptr) {
     delete lb;
   }
   m_size = 0;
@@ -794,7 +697,7 @@ LogBufferList::~LogBufferList()
 void
 LogBufferList::add(LogBuffer *lb)
 {
-  ink_assert(lb != NULL);
+  ink_assert(lb != nullptr);
 
   ink_mutex_acquire(&m_mutex);
   m_buffer_list.enqueue(lb);
@@ -814,7 +717,7 @@ LogBufferList::get()
 
   ink_mutex_acquire(&m_mutex);
   lb = m_buffer_list.dequeue();
-  if (lb != NULL) {
+  if (lb != nullptr) {
     m_size--;
     ink_assert(m_size >= 0);
   }
@@ -828,8 +731,8 @@ LogBufferList::get()
 LogEntryHeader *
 LogBufferIterator::next()
 {
-  LogEntryHeader *ret_val = NULL;
-  LogEntryHeader *entry   = (LogEntryHeader *)m_next;
+  LogEntryHeader *ret_val = nullptr;
+  LogEntryHeader *entry   = reinterpret_cast<LogEntryHeader *>(m_next);
 
   if (entry) {
     if (m_iter_entry_count < m_buffer_entry_count) {
